;Chess 2K19 Version 4f
;
;(c) Arkannoyed 2019
;
;Standard lines consist of 4 bits of data and a length of (Black or White) infill.
;These are created at their maximum possible length (28 bits) then squashed using the counter value
;to give the required length line, with the final bit being passed from one side to the other.
;Depending upon whether it is a Black or white piece, the sides are swapped.
;
;Data lines consist of 4 types
;1. long lines. These are lines of 18, 20 or 22 bits length, where the additional 6 bits over 16
;are stored within the index (first) byte, followed by 4 bytes of (W W XorB XorB) arranged data
;2. mini lines. Lines of 8 bits length or less, where left and right bytes are mirrored, so the
;4 bits are stored in the low 4 bits of the index byte. Same result for B & W versions.
;3. BLR-WLR lines. The Black and White versions of the line are different, but their Right and Left sides
;are mirrored versions of each other, where the low 5 bits of the index byte hold a XOR overlay to create
; the Black version from the White data. If the XOR ooverlay is 00000 then there is no change applied.
;2 byte line data.
;4. BWR-BWL lines. These lines are non-mirrored, but Black and White are the same, so the low 5 bits in the
;index are ADDed to the Left side to create the Right.
;2 byte line data.
;
;the line Buffers are HL for the left side and DE for the right. The Right side is auto masked and written byte-wise
;to the screen. The left side is reversed, so BIT 7 of H is the innermost in the line and they are written outwards (left)
;
;The last bit in the buffer is always '1'. All bits are RESet first before the data check is performed.
;then if we've a zero result from the operation, printing for the left side terminates.
;
;504 bytes total size
;466 bytes code and graphics data
;38 bytes for the board initialiser and its data
;
;Code 249 bytes
;Data 210 + 7 bytes
;

org 0fd8fh

init_board:
           ld de,0ffc0h                   ;initialise the board routine
           ld hl,data3s                   ;FFC0 is the start of the board matrix
lp1:
           xor a                          ;A=00
           ld c,(hl)                      ;save the contents of (HL)
           rrd                            ;piece code, lower nibble of (HL) into lower 4 bits of A
           ld b,(hl)                      ;remaining low nibble of (HL) into B as the counter
           inc b                          ;+1
           ld (hl),c                      ;restore (HL)
lp2:
           ld (de),a                      ;write the piece code to the board
           inc e                          ;next square
           djnz lp2                       ;loop
           inc hl                         ;next data byte
           jr nz,lp1                      ;if E=00 then end

chess_build:
           ld hl,09c0h                    ;initial value to calculate squares SCReen addr.
           ld c,l                         ;handy that the initial value of the screen calc is also init C!
main_loop:
           ld ix,0fe2dh                   ;IX = Graphics entry point address (base)

           ld a,c                         ;current square into A
           and 07h                        ;check row
           ld de,0ff30h                   ;DE=next row add value
           jr z,ovr_main                  ;if were still on the same row
           ld e,22h                       ;then add to go to the next square
           inc h
ovr_main:
           add hl,de                      ;add to adjust address to next square
           push hl                        ;save cumulative square address
get_height:
           ld e,c                         ;DE=current board address
           ld a,(de)                      ;A=piece occupying the current square
           rra                            ;lose bit 0 and /2 + add carry
           ld e,a                         ;to make DE=FF8x
           ld a,(de)                      ;get height from table at FF80h
           rl e                           ;restore current piece to E
           ld b,a                         ;B=height
get_len:
           inc d                          ;inc current line number
           push de                        ;line number (D) and piece (E) onto stack
           push bc                        ;height counter (B) and current square (C) onto stack
           push hl                        ;save the SCReen calc value in HL
scr_crt:
           ld a,h                         ;rotate H 3 times
           rlca                           ;to create the SCReen address
           rlca                           ;in HL
           rlca                           ;using A for the rotates is 4 ts faster!
           ld h,a                         ;restore into H
current_sq:
           ld a,c                         ;C=current square
           and 09h                        ;mask the bits that will indicate whether its a B or W square.
rotation:
           jp pe,main_call                ;if white square, jump to main call
                                          ;change to JP PO to effectively rotate the board
                                          ;90 degrees.
calc_line:
           ld a,d                         ;D=current line
           sub 11h                        ;are we above the height of the board square?
           jr nc,main_call                ;if so, skip drawing the line part of the square

           adc a,d                        ;add back 1 + D (line number)
           jr nc,ovr_e                    ;if we don't go above 00 then don't DEC or INVERT
           jr z,skp1                      ;if we're at the mid point to the square, longest line
           dec a                          ;then we can skip over the DEC A
skp1:
           cpl                            ;invert the result
ovr_e:
           and 0fh                        ;mask lower 4 bits

           inc a                          ;correct
           ld b,a                         ;Half line length in B used to find the start position to SET bits
           ld c,80h                       ;C=bit mask
add_line:
           push hl                        ;save HL position
addl_lp1:
           rlc c                          ;next bit left
           jr nc,addl_ovr1                ;
           dec l                          ;update HL if theres a carry
addl_ovr1:
           djnz addl_lp1                  ;loop
draw_line:
           add a,a                        ;x2 to get the full line length
           ld b,a                         ;into B again as the counter
addl_lp2:
           ld a,(hl)                      ;set the bit
           or c                           ;
           ld (hl),a                      ;merge bit with screen content
           rrc c                          ;nexr bit right
           jr nc,addl_ovr2                ;
           inc l                          ;next byte right
addl_ovr2:
           djnz addl_lp2                  ;loop

           pop hl                         ;restore HL SCReen address
main_call:
           sra e                          ;test E - piece no to see if a piece occupies the square.
           call nz,chess_2019             ;call main build routine if a piece occupies the square
nxt_line_up:
           pop hl                         ;restore SCReen calc vector
           ld bc,0ffe0h                   ;BC=addition vectors
           ld a,h                         ;next line up but shifted 3 bits rotationally right
           add a,c                        ;
           ld h,a                         ;
           jr c,piece_loop                ;Carry is ok, no further adjustment needed
           add hl,bc                      ;no carry means we alter L and H if,L carries.
piece_loop:
           pop bc
           pop de
           djnz get_len                   ;inner loop for pieces
end_check:
           pop hl                         ;restore square calc vector
           inc c                          ;next square
           jr nz,main_loop                ;repeat if needed (64 squares)

           ret                            ;ret
                                          ;107 bytes

pawn:
           db 9eh,0eeh ; (0e)                                  ;pawn j2
           db 57h
           db 2fh
           db 2eh
           db 1eh
           db 2eh
           db 1eh
           db 2fh
           db 0a8h,78h
           db 8dh
           db 84h
castle:
           db 98h,1eh ; (1b)                                       ;csl j2
           db 5dh
           db 5ch
           db 3ch
           db 5ch
           db 03h,8ah                                           ;knt sw3
           db 2ch
           db 5ch
           db 0a0h,67h
           db 0a0h,0a7h
           db 0a0h,9bh
           db 0a0h,92h                                          ;uses 1st line of base as last top line of csl
base:
           db 82h ; (2d)
           db 0a0h,0e4h
           db 0c0h,1dh,1dh,0e0h,0e0h
           db 1bh
           db 52h
           db 2ah
           db 52h
           db 0f6h,0f5h,0f0h,0fah,0fah
           db 0f6h,0eh,0eh,0ffh,0ffh
           db 53h
           db 14h

           db 01h,4dh ; (45)

           db 5ch ;47                                          ;pwn sw1
           db 5dh

           db 03h,5ah

           db 2dh ;                                         ;knt sw1
           db 5dh
           db 2dh ; (4d)
           db 2eh
           db 5eh
           db 01h,58h
           db 2eh
           db 5eh

           db 02h,5ah

           db 1fh ;                                           ;csl sw1
           db 2fh ;57

           db 1fh ; (58)                                           ;kng j1, qn j1, bsp j1
           db 2fh

           db 1fh ; (5a)                                           ;knt j1

           db 9fh,0ech ; (5b)                                      ;csl j1
           db 1dh
           db 1ch
           db 1ch

           db 01h,0eh

           db 9fh,0c6h ;62                                      ;pwn sw2

           db 02h,1bh

           db 03h,1ch
           db 9eh,3ch ;68                                      ;csl sw2, knt sw2
           db 17h
           db 1eh
           db 2eh

           db 04h,0d0h ;6d

           db 16h ; (6f)                                          ;bsp sw2
           db 5dh

           db 05h,80h
king:
           db 0a0h,1eh ; (73)                                     ;qn sw2
           db 0a0h,0e3h
           db 0a0h,7eh
           db 0a0h,0b4h
           db 89h
           db 8fh
           db 8fh
           db 8ah
           db 88h
queen:
           db 0a0h,0b6h ; (80)                                    ;qn j2
           db 0a0h,57h
           db 0a0h,6bh
           db 0a0h,0b2h
           db 05h,0ddh                                         ;qn sw3
knight:
           db 0e0h,17h,03h,0e8h,0a0h ; (8a)                         ;knt j3
           db 0e0h,0bh,06h,0f4h,40h
           db 0e0h,17h,0ch,68h,80h
           db 0e0h,0bh,31h,0b4h,00h
           db 0e4h,07h,43h,78h,00h
           db 0e6h,0bh,9dh,34h,00h
           db 0e3h,85h,61h,1ah,1ch
           db 0c3h,8bh,80h,34h,7eh
           db 0c6h,17h,00h,68h,0fdh
           db 0c4h,0eh,03h,0f0h,0f8h
           db 0c0h,2eh,65h,0d0h,0f8h
           db 0c0h,5ch,0dch,0a0h,0e0h
           db 0c0h,38h,68h,0c0h,80h
           db 0aeh,70h
           db 0ach,60h
           db 0b8h;0c0h
bishop:
           db 0c0h,0ech,0ch,10h,0c0h   ; (d0)                       ;bsp j2

           db 0aah,0bch
           db 0b0h,98h
           db 0aah,48h
           db 83h
           db 8bh
           db 8dh ; (dd)                                            ;qn j3
           db 8ch
           db 88h

           ;210 bytes 96 lines
data3s:                                      ;board setup data placed here so that it can share the last
           db 05h,07h,09h,0bh,0dh,09h,07h,05h;2 bytes (06h,04h) with the Build Routine.
           db 73h,0f0h,0f0h,72h              ;However, if a byte were sacrificed in the get height routine
           db 04h,06h,08h,0ah,0ch,08h        ;then the heights data can sit at FF00 directly before this instead.
chess_2019:
           ld b,04h                          ;pre-load B, which is also db 06h,04h, the end of the board setting data
           sbc a,a                           ;set B/W component according to BIT 0 of E
           ld c,a                            ;set B/W component in C, Black=FF, White=00
blanking:
           ld a,d                            ;test current line
                                             ;D just holds a number used for line check and masking + counter correction.
           cp b                              ;if 00-03 then RET, as we've not reached printing
           ret c                             ;of the base yet.

           push ix                           ;swap indexing the graphics data from IX to HL
           ex (sp),hl                        ;swap SCR address onto stack and IX from stack into HL
get_byte:
           ld a,(hl)                         ;get data byte in A
           inc l                             ;
           ld d,(hl)                         ;get D to use later or as the jump byte
           cp 07h                            ;test if A<07 indicating the target piece
           jr nc,test_7                      ;if +07 then continue processing data/std line
switch1:
           inc l                             ;
           cp e                              ;test against E, current piece
           jr nz,get_byte                    ;if not matching, then get next byte
switch2:
           ld l,d                            ;match found therefore we need to alter the addressing in HL using
           jr get_byte                       ;the second byte, now held in D, then repeat getting data
test_7:
           ld e,l                            ;adjust IX to match HL
           ld ixl,e                          ;

           add a,a                           ;test BIT 7 of A
           ld l,a                            ;save source in L

           jr c,data_line                    ;if type '1' line then jump to Data_Line builder

standard_line:
           and 0f0h                          ;mask Data part upper 4 bits
           ld d,a                            ;save in D
           xor l                             ;inverse mask - 0Fh to get counter
           add a,b                           ;ADD +04h - same as  SUB -0FCh
           ld b,a                            ;counter into B
           ld a,c                            ;B/W component into A
           or 0ch                            ;overlay add-in outer bits
           and 0fch                          ;mask 0FCh
           ld e,a                            ;into E
           xor d                             ;apply Data
           ld l,a                            ;into L
           ld d,c                            ;B/W component into D
           ld h,c                            ;B/W component into H
           inc c                             ;test C (00-01=W/FF-00=B)
           jr nz,st_lp1                      ;skip over swap if Black
st_lp0:
           ex de,hl                          ;swap buffers Left-Right
st_lp1:
           add hl,hl                         ;propagate the unwanted bits out of HL
           djnz st_lp0                       ;loop

           ld a,h                            ;replace BIT 7 of H
           rla                               ;with the last BIT carried
           rrca                              ;out of D

           jr bit4_res+1                     ;jump to printing line -1 (ld h,a)
data_line:
           add a,a                           ;test BIT 6
           jr c,long_line                    ;if its set then jump to Long lines.
mini_ln_tst:
           cp 3eh                            ;tset whether its a 1 byte mini line in case we need skip INC IX
           jr c,bit5_test                    ;
           inc ixl                           ;must be a 2 byte line, so set the data position correctly.
bit5_test:
           ld e,00h                          ;set the outer bytes to 00
           ld l,e                            ;L=00

           add a,a                           ;test BIT 5
           jr nc,bit4_test-1                 ;
bit5_set:
           add a,d                           ;BIT 5 was set, so add D to what remains in A
           jr bit4_res+1                     ;then skip to LD H,A prior to printing
long_line:
           and 0e0h                          ;mask upper 3 bits E0h
           ld e,a                            ;into E
lllp_00:
           ld a,(ix+00h)                     ;get White data
           and c                             ;mask with B/W component
           xor (ix+0feh)                     ;apply Black data switch
           ld d,h                            ;roll result from H into D. Pass 1 is null
           add hl,hl                         ;propagate 3 bits in L
           ld h,a                            ;B/W result into A
           inc ixl                           ;next
           djnz lllp_00                      ;loop x 3
           jr print_line                     ;second byte here is 07=RLCA
bit4_test:
           jr nc,bit4_res                    ;if bit ='0' then the data for BWRL is in the 4 bits remaining in A
bit4_set:
           rrca                              ;re-use bit to fill BIT 7, creating a left orientated mask
           and c                             ;mask with C BW byte to create 5 bit overlay mask of ccccc000
           xor d                             ;create the Black or White Left and Right buffer bytes
bit4_res:
           ld d,a                            ;load Right buffer
           ld h,a                            ;load Left buffer

print_line:
           ld c,80h                          ;bit mask for Left side print routine
           ex (sp),hl                        ;put HL Left buffer on stack and retrieve screen address
right:
           ld a,e                            ;test if E=00
           or a                              ;
           jr z,r_yes                        ;if E>00 then just copy D to screen.
r_no:
           ld (hl),d                         ;put full byte on screen, no need to mask
           inc l                             ;next L to far right byte
           ld d,e                            ;swap E into D so the routine that follows masks E instead
r_yes:
           ld a,d                            ;mask
           dec d                             ;and
           and d                             ;print D
           ld e,a                            ;
           xor d                             ;
           and (hl)                          ;merge with SCReen contents
           or e                              ;
           ld (hl),a                         ;place on SCReen

           res 0,l
get_lhs:
           pop de                            ;retrieve LHS data from stack
           ex de,hl                          ;swap addressing SCReen to DE, HL=data
left:
           rlc c                             ;next left bit
           jr nc,left_ovr                    ;
           dec e                             ;next left screen address
left_ovr:
           ld a,(de)                         ;get the SCReen byte in A
           or c                              ;set bit to 1
           xor c                             ;reset bit to 0. XOR C or SUB C does the same job.(A9 or 91) SBC A,C?(99)
           adc hl,hl                         ;propagate data in HL to get MSB
left_set:
           ld (de),a
           ret z                             ;if result of shift gave zero then RET
           jr nc,left                        ;if no carry then loop
           add a,c                           ;set bit to '1'. Either ADD A,C or OR C or XOR C does the same.(B1 or 81 or A9)
           jr left_set                       ;write result by looping back and as OR has cleared carry
                                             ;and the result will not be zero then we can continue
                                             ;the loop
                                             ;142 bytes
pieces_ht:
           db 11h,24h,29h,2ch,2fh,2fh,31h    ;pieces height data at ff80h









